SystemOrganization addCategory: #'XML-Writer-Core'!

!Character methodsFor: '*XML-Writer-Core' stamp: 'monty 12/2/2014 04:16'!
embedInMarkupWriter: aMarkupWriter
	self asString embedInMarkupWriter: aMarkupWriter! !

!BlockClosure methodsFor: '*XML-Writer-Core' stamp: 'monty 11/30/2014 06:05'!
embedInMarkupWriter: aMarkupWriter
	self cull: aMarkupWriter writer! !

!UndefinedObject methodsFor: '*XML-Writer-Core' stamp: 'monty 12/1/2014 08:02'!
embedInMarkupWriter: aMarkupWriter! !

!OrderPreservingStringDictionary methodsFor: '*XML-Writer-Core' stamp: 'monty 1/6/2015 19:57'!
writeAttributesOn: aTagWriter
	self keysDo: [:attributeName |
		aTagWriter
			writeAttributeName: attributeName
			value: (self at: attributeName ifAbsent: '')]! !

!OrderPreservingStringDictionary methodsFor: '*XML-Writer-Core' stamp: 'monty 1/6/2015 19:59'!
writeSortedAttributesOn: aTagWriter
	self keys sort do: [:attributeName |
		aTagWriter
			writeAttributeName: attributeName
			value: (self at: attributeName ifAbsent: '')]! !

Object subclass: #XMLExternalIDWriter
	instanceVariableNames: 'publicID systemID'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLExternalIDWriter commentStamp: 'monty 1/19/2014 22:07' prior: 0!
This class writes public and system IDs of external entities.!

!XMLExternalIDWriter methodsFor: 'testing' stamp: 'monty 11/30/2013 01:10'!
hasExternalID
	^ self publicID notEmpty or: [self systemID notEmpty]! !

!XMLExternalIDWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:53'!
publicID
	^ publicID ifNil: [publicID := '']! !

!XMLExternalIDWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:53'!
publicID: aPublicID
	publicID := aPublicID! !

!XMLExternalIDWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:54'!
systemID
	^ systemID ifNil: [systemID := '']! !

!XMLExternalIDWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:54'!
systemID: aSystemID
	systemID := aSystemID! !

!XMLExternalIDWriter methodsFor: 'writing' stamp: 'monty 2/9/2014 03:38'!
writeOn: aWriter
	self publicID
		ifNotEmpty: [:id |
			aWriter
				nextPutAll: 'PUBLIC';
				space;
				writeUnescapedString: id]
		ifEmpty: [
			self systemID
				ifNotEmpty: [aWriter nextPutAll: 'SYSTEM']].

	self systemID ifNotEmpty: [:id |
		aWriter
			space;
			writeUnescapedString: id].! !

Object subclass: #XMLMarkupWriter
	instanceVariableNames: 'writer isWriting'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLMarkupWriter commentStamp: 'JAAyer 7/12/2010 02:41' prior: 0!
This is a base class for markup writers. Subclasses override messages like #writePrologue and #writeEpilogue to write markup using the supplied XMLWriter object.!

XMLMarkupWriter subclass: #XMLCDataWriter
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLCDataWriter commentStamp: 'monty 12/2/2014 05:52' prior: 0!
A markup writer for <!![CDATA[...]]> sections. Will be output as regular escpaed strings instead if isCanonical is enbaled.!

!XMLCDataWriter methodsFor: 'accessing' stamp: 'monty 12/2/2014 00:22'!
string
	^ string ifNil: [string := '']! !

!XMLCDataWriter methodsFor: 'accessing' stamp: 'monty 12/1/2014 03:55'!
string: aString
	string := aString! !

!XMLCDataWriter methodsFor: 'private' stamp: 'monty 12/1/2014 03:59'!
writeBody
	self writer isCanonical
		ifTrue: [
			self parentMarkupWriter
				ifNotNil: [:parent | parent newStringMarkupBeingWritten].
			self writer writeEscapedContent: self string]
		ifFalse: [self writer nextPutAll: self string]! !

!XMLCDataWriter methodsFor: 'private' stamp: 'monty 12/1/2014 03:49'!
writeEpilogue
	self omitIfCanonical: [self writer nextPutAll: ']]>']! !

!XMLCDataWriter methodsFor: 'private' stamp: 'monty 12/1/2014 03:47'!
writePrologue
	self omitIfCanonical: [self writer nextPutAll: '<!![CDATA[']! !

XMLMarkupWriter subclass: #XMLCommentWriter
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLCommentWriter commentStamp: 'monty 12/2/2014 04:18' prior: 0!
A markup writer for <!!-- ... --> comments.!

!XMLCommentWriter methodsFor: 'accessing' stamp: 'monty 12/2/2014 00:22'!
string
	^ string ifNil: [string := '']! !

!XMLCommentWriter methodsFor: 'accessing' stamp: 'monty 12/1/2014 03:56'!
string: aString
	string := aString! !

!XMLCommentWriter methodsFor: 'private' stamp: 'monty 12/1/2014 04:12'!
writeBody
	self omitIfCanonicalInDTD: [self writer nextPutAll: self string]! !

!XMLCommentWriter methodsFor: 'private' stamp: 'monty 12/1/2014 04:12'!
writeEpilogue
	self omitIfCanonicalInDTD: [self writer nextPutAll: '-->']! !

!XMLCommentWriter methodsFor: 'private' stamp: 'monty 12/1/2014 23:39'!
writePrologue
	self omitIfCanonicalInDTD: [
		self writer
			formatBeforeComment: self;
			nextPutAll: '<!!--']! !

XMLMarkupWriter subclass: #XMLEmbeddedMarkupWriter
	instanceVariableNames: 'hasEmbeddedStringMarkup hasEmbeddedMarkupWriters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLEmbeddedMarkupWriter commentStamp: 'JAAyer 8/8/2010 17:29' prior: 0!
This class is a markup writer that supports embedding of child markup.!

XMLEmbeddedMarkupWriter subclass: #XMLDoctypeDeclarationWriter
	instanceVariableNames: 'root externalIDWriter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLDoctypeDeclarationWriter commentStamp: 'JAAyer 8/18/2010 21:27' prior: 0!
This class writes <!!DOCTYPE ...> declarations. You can set the root element, public ID and system ID using the messages under "accessing" before specifying any internal subset with #internalSubset: (which results in the declaration being written). If no public ID is supplied, then the declaration written will be of type "SYSTEM" rather than "PUBLIC" and will not have any Formal Public Identifier (FPI). The internal subset, if supplied, should consist of markup declarations generated with instances of XMLMarkupDeclarationWriter subclasses.!

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:54'!
externalIDWriter
	^ externalIDWriter ifNil: [externalIDWriter := XMLExternalIDWriter new]! !

!XMLDoctypeDeclarationWriter methodsFor: 'testing' stamp: 'monty 10/12/2014 21:12'!
isDoctypeWriter
	^ true! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
publicID
	^ self externalIDWriter publicID! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:47'!
publicID: aPublicID
	self externalIDWriter publicID: aPublicID! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:52'!
root
	^ root ifNil: ['']! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:52'!
root: aRootElementName
	root := aRootElementName! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
systemID
	^ self externalIDWriter systemID! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:47'!
systemID: aSystemID
	self externalIDWriter systemID: aSystemID! !

!XMLDoctypeDeclarationWriter methodsFor: 'embedding' stamp: 'monty 5/7/2015 18:55'!
withInternalSubset: aStringOrCollectionOrBlock
	self deprecated: 'use #write: instead'.

	self with: aStringOrCollectionOrBlock.! !

!XMLDoctypeDeclarationWriter methodsFor: 'embedding' stamp: 'monty 10/12/2014 20:49'!
writeBody: aStringOrCollectionOrBlock
	self omitIfCanonical: [super writeBody: aStringOrCollectionOrBlock]! !

!XMLDoctypeDeclarationWriter methodsFor: 'private' stamp: 'monty 12/1/2014 23:46'!
writeEmbeddedMarkupPrologue
	self omitIfCanonical: [
		self writer
			space;
			nextPut: $[;
			formatBeforeInternalSubset: self]! !

!XMLDoctypeDeclarationWriter methodsFor: 'private' stamp: 'monty 12/1/2014 23:45'!
writeEpilogue
	self omitIfCanonical: [
		self hasEmbeddedMarkup
			ifTrue: [
				self writer
					formatAfterInternalSubset: self;
					nextPut: $]].
		self writer
			nextPut: $>;
			formatAfterDoctypeDeclaration: self]! !

!XMLDoctypeDeclarationWriter methodsFor: 'private' stamp: 'monty 12/1/2014 23:44'!
writePrologue
	self omitIfCanonical: [
		self writer
			formatBeforeDoctypeDeclaration: self;
			nextPutAll: '<!!DOCTYPE';
			space;
			writeName: self root.

		self externalIDWriter hasExternalID
			ifTrue: [
				self writer space.
				self externalIDWriter writeOn: self writer]]! !

!XMLEmbeddedMarkupWriter methodsFor: 'testing' stamp: 'monty 12/1/2014 04:41'!
hasEmbeddedMarkup
	^ hasEmbeddedStringMarkup | hasEmbeddedMarkupWriters! !

!XMLEmbeddedMarkupWriter methodsFor: 'testing' stamp: 'monty 11/30/2014 06:17'!
hasEmbeddedMarkupWriters
	^ hasEmbeddedMarkupWriters! !

!XMLEmbeddedMarkupWriter methodsFor: 'testing' stamp: 'monty 11/30/2014 06:19'!
hasEmbeddedStringMarkup
	^ hasEmbeddedStringMarkup! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 12/1/2014 04:19'!
newMarkupWriterBeingPushed
	self isWriting
		ifTrue: [
			self hasEmbeddedMarkup
				ifFalse: [self writeEmbeddedMarkupPrologue].
			hasEmbeddedMarkupWriters := true]
		ifFalse: [self write]! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 12/1/2014 04:19'!
newStringMarkupBeingWritten
	self isWriting
		ifTrue: [
			self hasEmbeddedMarkup
				ifFalse: [self writeEmbeddedMarkupPrologue].
			hasEmbeddedStringMarkup := true]
		ifFalse: [self write]! !

!XMLEmbeddedMarkupWriter methodsFor: 'initialization' stamp: 'monty 11/30/2014 06:18'!
setWriter: aWriter
	super setWriter: aWriter.

	hasEmbeddedStringMarkup := false.
	hasEmbeddedMarkupWriters := false.! !

!XMLEmbeddedMarkupWriter methodsFor: 'embedding' stamp: 'monty 10/12/2014 19:46'!
with: aStringOrCollectionOrBlock
	self isWriting ifFalse: [
		self
			startWriting;
			writeBody: aStringOrCollectionOrBlock;
			finishWriting]! !

!XMLEmbeddedMarkupWriter methodsFor: 'writing' stamp: 'monty 10/12/2014 19:21'!
writeBody
	self writeBody: nil! !

!XMLEmbeddedMarkupWriter methodsFor: 'embedding' stamp: 'monty 12/1/2014 08:02'!
writeBody: aStringOrCollectionOrBlock
	aStringOrCollectionOrBlock embedInMarkupWriter: self! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 1/13/2014 23:27'!
writeEmbeddedMarkupPrologue! !

XMLEmbeddedMarkupWriter subclass: #XMLTagWriter
	instanceVariableNames: 'name xmlnsDeclarations attributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLTagWriter commentStamp: 'monty 1/13/2014 23:32' prior: 0!
This class generates XML tags for elements. You can specifiy the attributes of the start tag using the #attribute* messages, and you can add xmlns: namespace declarations using the #xmlns* messages. If you do not send #with:, or if you do but the argument is nil or a block that generates no markup, then only a single, empty tag in the form of "<empty />" will be generated, unless the writer object that created the tag writer object is configured to generate canonical XML, in which case an empty start/end tag pair like "<empty></empty>" will be generated.

You can supply child markup content using #with:. Namespace declarations will be ignored if a parent element has equivalent declarations (and they were thus already generated).!

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 14:54'!
attributeAt: aName
	^ self attributeAt: aName ifAbsent: ['']! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 12:38'!
attributeAt: aName ifAbsent: aBlock
	^ self attributes at: aName ifAbsent: aBlock! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 12:38'!
attributeAt: aName ifAbsentPut: aBlock
	^ self attributes at: aName ifAbsentPut: aBlock! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 12:38'!
attributeAt: aName put: aValue
	^ self attributes at: aName put: aValue! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 11/20/2014 19:48'!
attributes
	^ attributes ifNil: [attributes := OrderPreservingStringDictionary new]! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 11/20/2014 19:48'!
attributes: aDictionaryOrArray
	attributes := OrderPreservingStringDictionary newFrom: aDictionaryOrArray! !

!XMLTagWriter methodsFor: 'testing' stamp: 'monty 2/9/2014 04:19'!
escapesEmbeddedStrings
	^ true! !

!XMLTagWriter methodsFor: 'testing' stamp: 'monty 10/6/2014 23:30'!
hasAttributes
	^ self attributes notEmpty! !

!XMLTagWriter methodsFor: 'testing' stamp: 'monty 10/6/2014 23:30'!
includesAttribute: anAttribute
	^ self attributes includesKey: anAttribute! !

!XMLTagWriter methodsFor: 'testing' stamp: 'monty 10/12/2014 21:12'!
isTagWriter
	^ true! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:09'!
name
	^ name ifNil: ['']! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 6/20/2010 16:43'!
name: aName
	name := aName! !

!XMLTagWriter methodsFor: 'initialization' stamp: 'monty 3/24/2015 19:26'!
setAttributes: aDictionary
	attributes := aDictionary! !

!XMLTagWriter methodsFor: 'initialization' stamp: 'monty 3/24/2015 19:26'!
setXMLNSDeclarations: aDictionary 
	xmlnsDeclarations := aDictionary! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 11/22/2014 09:24'!
writeAttributeName: aName value: aValue 
	self writer
		space;
		writeName: aName
		escapedContentValue: aValue! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 1/6/2015 19:58'!
writeAttributes
	attributes
		ifNotNil: [
			self writer isCanonical
				ifTrue: [attributes writeSortedAttributesOn: self]
				ifFalse: [attributes writeAttributesOn: self]]! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 12/1/2014 23:36'!
writeEmbeddedMarkupPrologue
	self writer
		nextPut: $>;
		formatAfterStartTag: self! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 12/2/2014 05:45'!
writeEndTag
	self writer
		formatBeforeEndTag: self;
		nextPut: $<;
		nextPut: $/;
		writeName: self name;
		nextPut: $>! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 12/2/2014 05:45'!
writeEpilogue
	self hasEmbeddedMarkup
		ifTrue: [self writeEndTag]
		ifFalse: [
			(self writer isCanonical
				or: [self writer outputsSelfClosingTags not])
				ifTrue: [
					self writer nextPut: $>.
					self writeEndTag]
				ifFalse: [
					self writer
						space;
						nextPut: $/;
						nextPut: $>]].! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 3/24/2015 19:25'!
writePrologue
	self writer
		formatBeforeStartTag: self;
		nextPut: $<;
		writeName: self name.
	self
		writeXMLNSDeclarations;
		writeAttributes.! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 5/5/2015 19:20'!
writeXMLNS: aPrefix uri: aURI
	self writer
		space;
		writeName:
			(aPrefix isEmpty
				ifTrue: ['xmlns']
				ifFalse: ['xmlns:', aPrefix])
		escapedContentValue: aURI! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 3/24/2015 19:27'!
writeXMLNSDeclarations
	| checkForDuplicates parentMarkupWriter |

	xmlnsDeclarations
		ifNil: [^ self].
	checkForDuplicates := 
		self writer omitsRedundantXMLNSDeclarations
			and: [(parentMarkupWriter := self parentMarkupWriter) notNil
				and: [parentMarkupWriter isTagWriter]].
	self writer isCanonical
		ifTrue: [
			(xmlnsDeclarations keys reject: [:prefix |
				(checkForDuplicates
					and: [parentMarkupWriter xmlnsAt: prefix]) = (xmlnsDeclarations at: prefix)])
				sort;
				do: [:prefix |
					self
						writeXMLNS: prefix
						uri: (xmlnsDeclarations at: prefix)]]
		ifFalse: [
			xmlnsDeclarations keysDo: [:prefix | | uri |
				uri := xmlnsDeclarations at: prefix.
				(checkForDuplicates
					and: [(parentMarkupWriter xmlnsAt: prefix) = uri])
					ifFalse: [
						self
							writeXMLNS: prefix
							uri: uri]]].! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 20:59'!
xmlns
	"default namespace"
	^ self xmlnsDeclarations at: ''! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:47'!
xmlns: aURI
	"set default namespace"
	self xmlnsDeclarations at: '' put: aURI! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 19:47'!
xmlnsAt: aPrefix
	^ self xmlnsDeclarations at: aPrefix ifAbsent: ['']! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 19:47'!
xmlnsAt: aPrefix ifAbsent: aBlock
	^ self xmlnsDeclarations at: aPrefix ifAbsent: aBlock! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 22:29'!
xmlnsAt: aPrefix ifAbsentPut: aUriBlock
	^ self xmlnsDeclarations at: aPrefix ifAbsentPut: aUriBlock! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 19:47'!
xmlnsAt: aPrefix put: aValue
	^ self xmlnsDeclarations at: aPrefix put: aValue! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:55'!
xmlnsDeclarations
	^ xmlnsDeclarations
		ifNil: [xmlnsDeclarations := OrderPreservingStringDictionary new: 5]! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 11/20/2014 19:48'!
xmlnsDeclarations: aDictionaryOrArray
	"set xmlns decls using a dictionary or array of prefix/uri associations.
	empty prefix is default xmlns."
	xmlnsDeclarations := OrderPreservingStringDictionary newFrom: aDictionaryOrArray! !

XMLMarkupWriter subclass: #XMLEntityReferenceWriter
	instanceVariableNames: 'name isParameter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLEntityReferenceWriter commentStamp: 'monty 12/2/2014 00:38' prior: 0!
A writer for entity references.!

!XMLEntityReferenceWriter methodsFor: 'testing' stamp: 'monty 12/2/2014 00:33'!
isParameter
	^ isParameter ifNil: [isParameter := false]! !

!XMLEntityReferenceWriter methodsFor: 'accessing' stamp: 'monty 12/2/2014 00:32'!
isParameter: aBoolean
	isParameter := aBoolean! !

!XMLEntityReferenceWriter methodsFor: 'accessing' stamp: 'monty 12/2/2014 00:22'!
name
	^ name ifNil: [name := '']! !

!XMLEntityReferenceWriter methodsFor: 'accessing' stamp: 'monty 12/2/2014 00:42'!
name: aName
	name := aName! !

!XMLEntityReferenceWriter methodsFor: 'private' stamp: 'monty 12/2/2014 00:40'!
writeBody
	self writer writeName: self name! !

!XMLEntityReferenceWriter methodsFor: 'private' stamp: 'monty 12/2/2014 00:29'!
writeEpilogue
	self writer nextPut: $;! !

!XMLEntityReferenceWriter methodsFor: 'private' stamp: 'monty 12/2/2014 00:34'!
writePrologue
	self writer
		formatBeforeEntityReference: self;
		nextPut:
			(self isParameter
				ifTrue: [$%]
				ifFalse: [$&])! !

XMLMarkupWriter subclass: #XMLMarkupDeclarationWriter
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLMarkupDeclarationWriter commentStamp: 'JAAyer 8/18/2010 21:29' prior: 0!
This is a generic superclass for classes that generate DTD markup declarations--ELEMENT, ATTLIST, ENTITY and so on.!

XMLMarkupDeclarationWriter subclass: #XMLAttributeDeclarationWriter
	instanceVariableNames: 'attribute type defaultPragma defaultValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLAttributeDeclarationWriter commentStamp: 'monty 10/5/2014 18:03' prior: 0!
Writes ATTLIST declarations for attributes.!

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:55'!
attribute
	^ attribute ifNil: [attribute := '']! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:55'!
attribute: aName
	attribute := aName! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:27'!
declarationType
	^ 'ATTLIST'! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:15'!
defaultPragma
	^ defaultPragma ifNil: [defaultPragma := '']! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:17'!
defaultPragma: aDefaultPragma
	defaultPragma := aDefaultPragma! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:15'!
defaultValue
	^ defaultValue ifNil: [defaultValue := '']! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:17'!
defaultValue: aDefaultValue
	defaultValue := aDefaultValue! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:54'!
element
	^ self name! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:54'!
element: aName
	self name: aName! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 8/7/2010 13:51'!
type
	^ type ifNil: [type := 'CDATA']! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:30'!
type: aType
	type := aType! !

!XMLAttributeDeclarationWriter methodsFor: 'private' stamp: 'monty 10/12/2014 20:45'!
writeBody
	self omitIfCanonical: [
		self writer
			writeName: self attribute;
			space;
			writeName: self type.

		self
			writeDefaultPragma;
			writeDefaultValue]! !

!XMLAttributeDeclarationWriter methodsFor: 'private' stamp: 'monty 10/12/2014 20:46'!
writeDefaultPragma
	self defaultPragma
		ifNotEmpty: [:pragma |
			self writer space.

			(pragma first == $#)
				ifFalse: [self writer nextPut: $#].

			self writer nextPutAll: pragma]! !

!XMLAttributeDeclarationWriter methodsFor: 'private' stamp: 'monty 10/12/2014 20:46'!
writeDefaultValue
	self defaultValue
		ifNotEmpty: [:value |
			self writer
				space;
				writeEscapedContentString: value]! !

XMLMarkupDeclarationWriter subclass: #XMLElementDeclarationWriter
	instanceVariableNames: 'contentModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLElementDeclarationWriter commentStamp: 'monty 10/5/2014 18:04' prior: 0!
Writes ELEMENT declarations. 
!

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 8/7/2010 13:52'!
contentModel
	^ contentModel ifNil: [contentModel := '']! !

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 15:11'!
contentModel: aContentModelString
	contentModel := aContentModelString! !

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:27'!
declarationType
	^ 'ELEMENT'! !

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:17'!
element
	^ self name! !

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:17'!
element: aName
	self name: aName! !

!XMLElementDeclarationWriter methodsFor: 'private' stamp: 'monty 10/12/2014 20:22'!
writeBody
	self omitIfCanonical: [self writer nextPutAll: self contentModel]! !

XMLMarkupDeclarationWriter subclass: #XMLEntityDeclarationWriter
	instanceVariableNames: 'isParameter replacement externalIDWriter ndata'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLEntityDeclarationWriter commentStamp: 'monty 10/5/2014 18:04' prior: 0!
Writes general and parameter ENTITY declarations.!

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 11/20/2013 23:41'!
declarationType
	^ self isParameter
		ifTrue: ['ENTITY %']
		ifFalse: ['ENTITY']! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:56'!
externalIDWriter
	^ externalIDWriter ifNil: [externalIDWriter := XMLExternalIDWriter new]! !

!XMLEntityDeclarationWriter methodsFor: 'testing' stamp: 'monty 11/13/2013 19:36'!
isParameter
	^ isParameter ifNil: [isParameter := false]! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 11/13/2013 19:36'!
isParameter: aBoolean
	isParameter := aBoolean! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 8/7/2010 14:45'!
ndata
	^ ndata ifNil: [ndata := '']! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 17:37'!
ndata: aNotationName
	ndata := aNotationName! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
publicID
	^ self externalIDWriter publicID! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:47'!
publicID: aPublicID
	self externalIDWriter publicID: aPublicID! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 8/7/2010 14:44'!
replacement
	^ replacement ifNil: [replacement := '']! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 17:35'!
replacement: aReplacement
	replacement := aReplacement! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
systemID
	^ self externalIDWriter systemID! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:47'!
systemID: aSystemID
	self externalIDWriter systemID: aSystemID! !

!XMLEntityDeclarationWriter methodsFor: 'private' stamp: 'monty 10/12/2014 20:19'!
writeBody
	self omitIfCanonical: [
		self replacement
			ifNotEmpty: [self writer writeEscapedEntityValueString: self replacement]
			ifEmpty: [
				self externalIDWriter writeOn: self writer.
				(self isParameter not and: [self ndata notEmpty])
					ifTrue: [
						self writer
							space;
							nextPutAll: 'NDATA';
							space;
							writeName: self ndata]]]! !

!XMLMarkupDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:26'!
declarationType
	self subclassResponsibility! !

!XMLMarkupDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:43'!
name
	^ name ifNil: ['']! !

!XMLMarkupDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 6/20/2010 17:02'!
name: aName
	name := aName! !

!XMLMarkupDeclarationWriter methodsFor: 'private' stamp: 'monty 10/12/2014 20:16'!
writeEpilogue
	self omitIfCanonical: [self writer nextPut: $>]! !

!XMLMarkupDeclarationWriter methodsFor: 'private' stamp: 'monty 12/1/2014 23:40'!
writePrologue
	self omitIfCanonical: [
		self writer
			formatBeforeDeclaration: self;
			nextPutAll: '<!!';
			writeName: self declarationType;
			space;
			writeName: self name;
			space]! !

XMLMarkupDeclarationWriter subclass: #XMLNotationDeclarationWriter
	instanceVariableNames: 'externalIDWriter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLNotationDeclarationWriter commentStamp: 'monty 10/5/2014 18:04' prior: 0!
Writes NOTATION declarations.!

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:27'!
declarationType
	^ 'NOTATION'! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:49'!
externalIDWriter
	^ externalIDWriter ifNil: [externalIDWriter := XMLExternalIDWriter new]! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
publicID
	^ self externalIDWriter publicID! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/22/2015 02:19'!
publicID: aPublicID
	self externalIDWriter publicID: aPublicID! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
systemID
	^ self externalIDWriter systemID! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:47'!
systemID: aSystemID
	self externalIDWriter systemID: aSystemID! !

!XMLNotationDeclarationWriter methodsFor: 'private' stamp: 'monty 10/12/2014 20:19'!
writeBody
	self omitIfCanonical: [self externalIDWriter writeOn: self writer]! !

!XMLMarkupWriter class methodsFor: 'instance creation' stamp: 'monty 11/30/2014 03:31'!
writer: aWriter
	^ self new setWriter: aWriter! !

!XMLMarkupWriter methodsFor: 'converting' stamp: 'monty 5/6/2015 17:05'!
asString
	"for GS compatibility"
	^ self printString! !

!XMLMarkupWriter methodsFor: 'writing' stamp: 'monty 5/6/2015 16:31'!
close
	self writer close! !

!XMLMarkupWriter methodsFor: 'accessing' stamp: 'monty 5/6/2015 16:31'!
contents
	^ self writer contents! !

!XMLMarkupWriter methodsFor: 'writing' stamp: 'monty 12/5/2014 05:55'!
finishWriting
	self hasEmbeddedMarkupWriters
		ifTrue: [self writer writeUnwrittenMarkupWritersEmbeddedIn: self].
	self writeEpilogue.
	isWriting := false.

	self writer popActiveMarkupWriter.! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 1/14/2014 00:18'!
hasEmbeddedMarkup
	^ false! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 10/4/2014 04:21'!
hasEmbeddedMarkupWriters
	^ false! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 12/1/2014 05:19'!
hasEmbeddedStringMarkup
	^ false! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 10/12/2014 21:11'!
isDoctypeWriter
	^ false! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 12/1/2014 09:53'!
isEmbeddedWithStringMarkup
	| parent |

	^ (parent := self parentMarkupWriter) notNil
		and: [parent hasEmbeddedStringMarkup]! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 10/12/2014 21:11'!
isTagWriter
	^ false! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 11/30/2014 03:32'!
isWriting
	^ isWriting! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'monty 1/16/2014 23:00'!
newMarkupWriterBeingPushed
	self write! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'monty 11/30/2014 06:21'!
newStringMarkupBeingWritten
	self write! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'monty 10/12/2014 20:14'!
omitIfCanonical: aBlock
	self writer isCanonical
		ifFalse: [aBlock value]! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'monty 12/1/2014 04:10'!
omitIfCanonicalInDTD: aBlock
	| parent |

	(self writer isCanonical
		and: [(parent := self parentMarkupWriter) notNil
			and: [parent isDoctypeWriter]])
		ifFalse: [aBlock value]! !

!XMLMarkupWriter methodsFor: 'accessing' stamp: 'monty 12/1/2014 04:07'!
parentMarkupWriter
	^ self writer parentMarkupWriterOf: self! !

!XMLMarkupWriter methodsFor: 'printing' stamp: 'JAAyer 7/21/2010 09:26'!
printOn: aStream
	self writer printOn: aStream! !

!XMLMarkupWriter methodsFor: 'initialization' stamp: 'monty 11/30/2014 03:32'!
setWriter: aWriter
	writer := aWriter.
	isWriting := false.! !

!XMLMarkupWriter methodsFor: 'writing' stamp: 'monty 10/5/2014 15:43'!
startWriting
	isWriting := true.
	self writePrologue.! !

!XMLMarkupWriter methodsFor: 'writing' stamp: 'monty 12/5/2014 04:30'!
write
	self isWriting
		ifFalse: [
			self
				startWriting;
				writeBody;
				finishWriting]! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'JAAyer 7/26/2010 12:12'!
writeBody! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'JAAyer 6/20/2010 12:08'!
writeEpilogue! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'JAAyer 6/20/2010 12:08'!
writePrologue! !

!XMLMarkupWriter methodsFor: 'accessing' stamp: 'JAAyer 6/18/2010 22:05'!
writer
	^ writer! !

XMLMarkupWriter subclass: #XMLPIWriter
	instanceVariableNames: 'target data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLPIWriter commentStamp: 'JAAyer 7/11/2010 05:11' prior: 0!
This class generates processing instructions.!

!XMLPIWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:10'!
data
	^ data ifNil: ['']! !

!XMLPIWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 12:39'!
data: aString
	data := aString! !

!XMLPIWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:10'!
target
	^ target ifNil: ['']! !

!XMLPIWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:50'!
target: aString
	target := aString! !

!XMLPIWriter methodsFor: 'private' stamp: 'monty 10/12/2014 21:15'!
writeBody
	self omitIfCanonicalInDTD: [
		self writer
			space;
			nextPutAll: self data]! !

!XMLPIWriter methodsFor: 'private' stamp: 'monty 10/12/2014 21:15'!
writeEpilogue
	self omitIfCanonicalInDTD: [self writer nextPutAll: '?>']! !

!XMLPIWriter methodsFor: 'private' stamp: 'monty 12/1/2014 23:41'!
writePrologue
	self omitIfCanonicalInDTD: [
		self writer
			formatBeforePI: self;
			nextPutAll: '<?';
			writeName: self target]! !

XMLMarkupWriter subclass: #XMLXMLDeclarationWriter
	instanceVariableNames: 'encoding version standalone'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLXMLDeclarationWriter commentStamp: '<historical>' prior: 0!
This class generates <?xml ...?> declarations.!

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:11'!
encoding
	^ encoding ifNil: [encoding := 'UTF-8']! !

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:13'!
encoding: anEncoding
	encoding := anEncoding! !

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'monty 11/19/2013 05:43'!
standalone
	^ standalone ifNil: [standalone := false]! !

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/16/2014 19:27'!
standalone: aBooleanOrString
	aBooleanOrString isString
		ifTrue: [standalone := aBooleanOrString = 'yes']
		ifFalse: [standalone := aBooleanOrString].! !

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:11'!
version
	^ version ifNil: [version := '1.0']! !

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:30'!
version: aVersion
	| versionString |

	version := "avoid first: for portability"
		(versionString := (aVersion ifNil: ['1.0']) asString)
			copyFrom: 1
			to: (3 min: versionString size)! !

!XMLXMLDeclarationWriter methodsFor: 'private' stamp: 'monty 1/7/2015 00:06'!
writeBody
	self omitIfCanonical: [
		self writer
			space;
			writeName: 'version'
			unescapedValue: (self version ifEmpty: ['1.0']).
		self encoding
			ifNotEmpty: [:encodingName |
				self writer
					space;
					writeName: 'encoding'
					unescapedValue: encodingName].
		self standalone
			ifTrue: [
				self writer
					space;
					writeName: 'standalone' unescapedValue: 'yes']]! !

!XMLXMLDeclarationWriter methodsFor: 'private' stamp: 'monty 10/12/2014 20:20'!
writeEpilogue
	self omitIfCanonical: [self writer nextPutAll: '?>']! !

!XMLXMLDeclarationWriter methodsFor: 'private' stamp: 'monty 3/24/2015 19:34'!
writePrologue
	self omitIfCanonical: [
		self writer
			formatBeforeXMLDeclaration: self;
			nextPutAll: '<?xml']! !

Object subclass: #XMLWriter
	instanceVariableNames: 'stream markupWriterStack formatter lineBreak contentEscapes contentStringEscapes isEscaping isCanonical outputsSelfClosingTags omitsRedundantXMLNSDeclarations'
	classVariableNames: 'ContentEscapes ContentNonEntityEscapes ContentStringEscapes ContentStringNonEntityEscapes EntityValueEscapes'
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLWriter commentStamp: 'monty 5/6/2015 21:26' prior: 0!
This class allows you to generate well-formed XML documents using an API similar to Seaside's canvas and tag brush API.

Markup can be created by sending an instance any of the messages under "writing." The messages that take no arguments return an instance of an XMLMarkupWriter subclass. These objects can be configured with messages like #name: and #attributeAt:put:. Sending them #write or #with: (if they can have embedded markup) causes the markup writer's configuration to be committed and its markup to be written to the output stream.

For markup writers that can have embedded markup, the argument to #with: can be a string, collection (of strings and blocks), a block, or nil. A block passed in can optionally take an argument, which will be the XMLWriter. Creating additional non-embedded markup will cause any unwritten markup from a previous markup writer to be written, as will sending an XMLWriter or markup writer #write, #contents, #printOn:, #asString, or #close.

Here is an example (evaluate with cmd-p):

	| writer |
	writer := XMLWriter new.
	writer
		enablePrettyPrinting;
		xml;
		comment: 'an example'.
	writer tag
		name: 'example:user';
		xmlnsAt: 'example' put: 'urn://example';
		with: [
			writer
				tag: 'username' with: 'jsmith';
				tag: 'real-name' with: 'John Smith';
				tag: 'age' with: '25';
				tag: 'id' with: '101']

If the last message sent isn't a serialization message like #with: or #tag:with:, you may need to send #write to force writing any unwritten markup:

	String streamContents: [:stream |
		(XMLWriter on: stream)
			tag: 'first';
			tag: 'second';
			write].

The second tag forces the first to be written, but the second won't be written unless #write is sent to its tag writer or the XMLWriter.

The #writeWith: message evaluates a block and automatically sends #write after, similar to how #fileNamed:do: will automatically send #close for you:

	String streamContents: [:stream |
		(XMLWriter  on: stream)
			writeWith: [:writer |
				writer
					tag: 'first'
					with: [writer tag: 'second'].
				writer tag: 'third']].!

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 1/5/2015 07:27'!
initialize
	"self initialize"

	self
		initializeContentEscapes;
		initializeContentNonEntityEscapes;
		initializeContentStringEscapes;
		initializeContentStringNonEntityEscapes;
		initializeEntityValueEscapes! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 12/13/2014 10:49'!
initializeContentEscapes
	(ContentEscapes := Array new: 256)
		at: $& asciiValue put: '&amp;';
		at: $< asciiValue put: '&lt;';
		at: $> asciiValue put: '&gt;'; "required to escape ]]> in PCDATA"
		at: Character cr asciiValue put: #LineBreak;
		at: Character lf asciiValue put: #LineBreak! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 1/5/2015 07:26'!
initializeContentNonEntityEscapes
	(ContentNonEntityEscapes := ContentEscapes copy)
		at: $& asciiValue
		put: nil! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 12/1/2014 09:27'!
initializeContentStringEscapes
	(ContentStringEscapes := Array new: 256)
		at: $& asciiValue put: '&amp;';
		at: $< asciiValue put: '&lt;';
		at: $" asciiValue put: '&quot;';
		at: Character cr asciiValue put: #LineBreak;
		at: Character lf asciiValue put: #LineBreak! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 1/5/2015 07:26'!
initializeContentStringNonEntityEscapes
	(ContentStringNonEntityEscapes := ContentStringEscapes copy)
		at: $& asciiValue
		put: nil! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 12/13/2014 10:50'!
initializeEntityValueEscapes
	(EntityValueEscapes := Array new: 256)
		at: $" asciiValue put: '&#34;';
		at: $% asciiValue put: '&#37;';
		at: Character cr asciiValue put: #LineBreak;
		at: Character lf asciiValue put: #LineBreak! !

!XMLWriter class methodsFor: 'instance creation' stamp: 'JAAyer 7/27/2010 13:01'!
new
	^ self on: (String new: 512) writeStream! !

!XMLWriter class methodsFor: 'instance creation' stamp: 'monty 5/6/2015 01:29'!
on: aStream
	^ self basicNew initialize on: aStream! !

!XMLWriter class methodsFor: 'instance creation' stamp: 'monty 5/6/2015 01:26'!
on: aStream formatter: aFormatter
	^ (self on: aStream) formatter: aFormatter! !

!XMLWriter class methodsFor: 'writing' stamp: 'monty 10/26/2014 10:17'!
writeWith: aBlock
	^ self new
		writeWith: aBlock;
		contents! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 10/12/2014 06:57'!
% aParameterEntityReference
	self paramEntityRef: aParameterEntityReference! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 10/12/2014 06:57'!
& anEntityReference
	self entityRef: anEntityReference! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 10/27/2014 08:19'!
activeMarkupWriter
	"for speed"
	^ markupWriterStack size > 0
		ifTrue: [markupWriterStack at: markupWriterStack size]
		ifFalse: [nil]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/5/2014 01:52'!
activeMarkupWriterDo: aBlock
	"This evaluates aBlock for the active (top) markup writer until
	it is nil or is the same (still active) writer after evaluating aBlock"
	| activeMarkupWriter previousActiveMarkupWriter |

	[(activeMarkupWriter := self activeMarkupWriter) isNil
		or: [activeMarkupWriter == previousActiveMarkupWriter]]
		whileFalse: [
			aBlock value: activeMarkupWriter.
			previousActiveMarkupWriter := activeMarkupWriter]! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 12/2/2014 00:05'!
activeTagWriter
	self markupWriterStack reverseDo: [:each |
		each isTagWriter
			ifTrue: [^ each]].
	^ nil.! !

!XMLWriter methodsFor: 'converting' stamp: 'monty 5/6/2015 17:05'!
asString
	"for GS compatibility"
	^ self printString! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 5/6/2015 01:33'!
atBeginning
	"overridden in GS version to use Stream>>atBeginning"
	^ stream position = 0! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 1/13/2014 23:52'!
attributeDeclaration
	^ self pushNewMarkupWriter: XMLAttributeDeclarationWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/1/2014 04:05'!
cdata
	^ self pushNewMarkupWriter: XMLCDataWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/1/2014 04:05'!
cdata: aString
	self cdata
		string: aString;
		write! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 1/13/2014 23:10'!
close
	self write.
	stream close.! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/1/2014 04:04'!
comment
	^ self pushNewMarkupWriter: XMLCommentWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/1/2014 04:04'!
comment: aString
	self comment
		string: aString;
		write! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 1/13/2014 23:10'!
contents
	self write.
	^ stream contents.! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/1/2014 06:32'!
defaultLineBreak
	^ String cr! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 1/13/2014 23:52'!
doctype
	^ self pushNewMarkupWriter: XMLDoctypeDeclarationWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 1/13/2014 23:52'!
elementDeclaration
	^ self pushNewMarkupWriter: XMLElementDeclarationWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 11/30/2014 05:15'!
enablePrettyPrinting
	self formatter: (XMLPrettyPrintingWriterFormatter writer: self)! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 1/13/2014 23:52'!
entityDeclaration
	^ self pushNewMarkupWriter: XMLEntityDeclarationWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/2/2014 02:07'!
entityRef: aName
	(self pushNewMarkupWriter: XMLEntityReferenceWriter)
		name: aName;
		write! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 1/5/2015 07:45'!
escapesContentEntityReferences
	^ (contentEscapes at: $& asciiValue) notNil
		and: [(contentStringEscapes at: $& asciiValue) notNil]! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 1/5/2015 07:43'!
escapesContentEntityReferences: aBoolean
	aBoolean
		ifTrue: [
			contentEscapes := ContentEscapes.
			contentStringEscapes := ContentStringEscapes]
		ifFalse: [
			contentEscapes := ContentNonEntityEscapes.
			contentStringEscapes := ContentStringNonEntityEscapes]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:45'!
formatAfterDoctypeDeclaration: aDoctypeDeclarationWriter
	formatter
		ifNotNil: [formatter formatAfterDoctypeDeclaration: aDoctypeDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:42'!
formatAfterInternalSubset: aDoctypeDeclarationWriter
	formatter
		ifNotNil: [formatter formatAfterInternalSubset: aDoctypeDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:34'!
formatAfterStartTag: aTagWriter
	formatter
		ifNotNil: [formatter formatAfterStartTag: aTagWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:39'!
formatBeforeComment: aCommentWriter
	formatter
		ifNotNil: [formatter formatBeforeComment: aCommentWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:39'!
formatBeforeDeclaration: aDeclarationWriter
	formatter
		ifNotNil: [formatter formatBeforeDeclaration: aDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:44'!
formatBeforeDoctypeDeclaration: aDoctypeDeclarationWriter
	formatter
		ifNotNil: [formatter formatBeforeDoctypeDeclaration: aDoctypeDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:34'!
formatBeforeEndTag: aTagWriter
	formatter
		ifNotNil: [formatter formatBeforeEndTag: aTagWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/2/2014 05:41'!
formatBeforeEntityReference: anEntityReferenceWriter
	formatter
		ifNotNil: [formatter formatBeforeEntityReference: anEntityReferenceWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:46'!
formatBeforeInternalSubset: aDoctypeDeclarationWriter
	formatter
		ifNotNil: [formatter formatBeforeInternalSubset: aDoctypeDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:41'!
formatBeforePI: aPiWriter
	formatter
		ifNotNil: [formatter formatBeforePI: aPiWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:34'!
formatBeforeStartTag: aTagWriter
	formatter
		ifNotNil: [formatter formatBeforeStartTag: aTagWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 3/24/2015 19:34'!
formatBeforeXMLDeclaration: anXMLDeclarationWriter
	formatter
		ifNotNil: [formatter formatBeforeXMLDeclaration: anXMLDeclarationWriter]! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 11/30/2014 04:03'!
formatter
	^ formatter! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 11/30/2014 05:14'!
formatter: aFormatter
	formatter := aFormatter! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 11/22/2014 09:39'!
isCanonical
	^ isCanonical! !

!XMLWriter methodsFor: 'accessing' stamp: 'JAAyer 6/20/2010 20:43'!
isCanonical: aBoolean
	isCanonical := aBoolean! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 11/22/2014 09:41'!
isEscaping
	^ isEscaping! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 2/9/2014 03:47'!
isEscaping: aBoolean
	isEscaping := aBoolean! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 12/1/2014 06:32'!
lineBreak
	^ lineBreak! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 12/1/2014 06:32'!
lineBreak: aCharacterOrString
	lineBreak := aCharacterOrString asString! !

!XMLWriter methodsFor: 'private' stamp: 'monty 10/27/2014 08:19'!
markupWriterStack
	^ markupWriterStack! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/5/2014 03:51'!
newStringMarkup
	self activeMarkupWriterDo: [:each | each newStringMarkupBeingWritten]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/5/2014 04:07'!
nextPut: aCharacter
	^ stream nextPut: aCharacter! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/5/2014 04:07'!
nextPutAll: aString
	^ stream nextPutAll: aString! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 1/13/2014 23:52'!
notationDeclaration
	^ self pushNewMarkupWriter: XMLNotationDeclarationWriter! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 3/24/2015 19:27'!
omitsRedundantXMLNSDeclarations
	^ omitsRedundantXMLNSDeclarations! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:27'!
omitsRedundantXMLNSDeclarations: aBoolean
	omitsRedundantXMLNSDeclarations := aBoolean! !

!XMLWriter methodsFor: 'initialization' stamp: 'monty 5/6/2015 01:27'!
on: aStream
	stream := aStream.
	markupWriterStack := OrderedCollection new.
	lineBreak := self defaultLineBreak.
	contentEscapes := ContentEscapes.
	contentStringEscapes := ContentStringEscapes.
	isEscaping := true.
	isCanonical := false.
	outputsSelfClosingTags := true.
	omitsRedundantXMLNSDeclarations := true.! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 11/22/2014 09:39'!
outputsSelfClosingTags
	^ outputsSelfClosingTags! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 10/12/2014 21:02'!
outputsSelfClosingTags: aBoolean
	outputsSelfClosingTags := aBoolean! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/2/2014 00:38'!
paramEntityRef: aName
	(self pushNewMarkupWriter: XMLEntityReferenceWriter)
		isParameter: true;
		name: aName;
		write! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 12/2/2014 23:30'!
parentMarkupWriterOf: aMarkupWriter
	self markupWriterStack size to: 2 by: -1 do: [:i |
		(self markupWriterStack at: i) == aMarkupWriter
			ifTrue: [^ self markupWriterStack at: i - 1]].
	^ nil.! !

!XMLWriter methodsFor: 'writing' stamp: 'JAAyer 7/21/2010 01:11'!
pcdata: aString
	self string: aString! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 1/13/2014 23:52'!
pi
	^ self pushNewMarkupWriter: XMLPIWriter! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/13/2014 10:53'!
popActiveMarkupWriter
	^ self markupWriterStack removeLast! !

!XMLWriter methodsFor: 'printing' stamp: 'monty 12/5/2014 03:51'!
printOn: aStream
	aStream nextPutAll: self contents! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/4/2014 01:25'!
pushNewMarkupWriter: aMarkupWriterClass
	| newMarkupWriter |

	newMarkupWriter := aMarkupWriterClass writer: self.
	self activeMarkupWriterDo: [:each | each newMarkupWriterBeingPushed].

	^ self markupWriterStack addLast: newMarkupWriter.! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 11/30/2014 06:15'!
raw: aString
	self
		newStringMarkup;
		nextPutAll: aString! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/22/2014 09:31'!
space
	stream space! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/13/2014 23:14'!
stream
	^ stream! !

!XMLWriter methodsFor: 'private' stamp: 'monty 5/6/2015 16:05'!
stream: aStream
	self on: aStream! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/5/2014 03:36'!
string: aString
	self
		newStringMarkup;
		writeEscapedContent: aString! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 1/13/2014 23:52'!
tag
	^ self pushNewMarkupWriter: XMLTagWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'JAAyer 6/26/2010 21:47'!
tag: aName
	^ self tag name: aName! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 10/6/2014 23:36'!
tag: aName attributes: aDictionaryOrArray
	^ (self tag: aName)
		attributes: aDictionaryOrArray! !

!XMLWriter methodsFor: 'writing' stamp: 'JAAyer 11/8/2010 02:53'!
tag: aName attributes: aDictionaryOrArray with: aStringOrCollectionOrBlock
	^ (self tag: aName)
		attributes: aDictionaryOrArray;
		with: aStringOrCollectionOrBlock! !

!XMLWriter methodsFor: 'writing' stamp: 'JAAyer 11/8/2010 02:54'!
tag: aName with: aStringOrCollectionOrBlock
	^ (self tag: aName) with: aStringOrCollectionOrBlock! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 1/11/2015 23:53'!
write
	self activeMarkupWriterDo: [:each |
		each isWriting
			ifFalse: [each write]]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/5/2015 07:23'!
write: aString escapedWith: anEscapeTable
	self isEscaping
		ifTrue: [| tableSize |
			tableSize := anEscapeTable size.
			"to:do: for performance"
			1 to: aString size do: [:i | | nextChar asciiValue |
				((asciiValue := (nextChar := aString at: i) asciiValue) > 0
					and: [asciiValue <= tableSize
						and: [(anEscapeTable at: asciiValue) notNil]])
					ifTrue: [| replacement |
						stream nextPutAll:
							((replacement := anEscapeTable at: asciiValue) == #LineBreak
								ifTrue: [self lineBreak]
								ifFalse: [replacement])]
					ifFalse: [stream nextPut: nextChar]]]
		ifFalse: [stream nextPutAll: aString]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/5/2015 07:40'!
writeEscapedContent: aString
	self
		write: aString
		escapedWith: contentEscapes! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/5/2015 07:40'!
writeEscapedContentString: aString
	self
		nextPut: $";
		write: aString escapedWith: contentStringEscapes;
		nextPut: $"! !

!XMLWriter methodsFor: 'private' stamp: 'monty 2/9/2014 04:30'!
writeEscapedEntityValueString: aString
	self
		nextPut: $";
		write: aString escapedWith: EntityValueEscapes;
		nextPut: $"! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/13/2014 10:42'!
writeName: aString
	stream nextPutAll: aString! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/13/2014 10:41'!
writeName: aName escapedContentValue: aValue
	self
		writeName: aName;
		nextPut: $=;
		writeEscapedContentString: aValue! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/13/2014 10:42'!
writeName: aName unescapedValue: aValue
	self
		writeName: aName;
		nextPut: $=;
		writeUnescapedString: aValue! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/13/2014 11:04'!
writeUnescapedString: aString
	| quote |

	quote :=
		(aString includes: $")
			ifTrue: [$']
			ifFalse: [$"].
	self nextPut: quote.
	aString do: [:each |
		"strip delimiter because it can't be escaped with an entity in this type of string"
		each == quote
			ifFalse: [self nextPut: each]].
	self nextPut: quote.! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/11/2015 23:53'!
writeUnwrittenMarkupWritersEmbeddedIn: aMarkupWriter
	self activeMarkupWriterDo: [:each |
		each == aMarkupWriter
			ifTrue: [^ self].
		each isWriting
			ifFalse: [each write]]! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 1/14/2014 01:34'!
writeWith: aBlock
	aBlock cull: self.
	self write.! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 3/24/2015 19:25'!
xml
	^ self pushNewMarkupWriter: XMLXMLDeclarationWriter! !

Object subclass: #XMLWriterFormatter
	instanceVariableNames: 'writer indentString currentIndentLevel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLWriterFormatter commentStamp: 'monty 12/3/2014 08:05' prior: 0!
An abstract class for formatters used to decorate markup writers.!

XMLWriterFormatter subclass: #XMLPrettyPrintingWriterFormatter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLPrettyPrintingWriterFormatter commentStamp: 'monty 5/5/2015 19:19' prior: 0!
A formatter that inserts ignorable whitespace to make the XML more readable.!

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:45'!
formatAfterDoctypeDeclaration: aDoctypeDeclarationWriter
	self writeLineBreak! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:43'!
formatAfterInternalSubset: aDoctypeDeclarationWriter
	aDoctypeDeclarationWriter hasEmbeddedMarkup
		ifTrue: [
			self
				outdent;
				writeLineBreak]! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:33'!
formatAfterStartTag: aTagWriter
	self indent! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:39'!
formatBeforeComment: aCommentWriter
	aCommentWriter isEmbeddedWithStringMarkup
		ifFalse: [self writeLineBreak]! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:40'!
formatBeforeDeclaration: aDeclarationWriter
	self writeLineBreak! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:43'!
formatBeforeDoctypeDeclaration: aDoctypeDeclarationWriter
	self writeLineBreak! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:33'!
formatBeforeEndTag: aTagWriter
	aTagWriter hasEmbeddedMarkup
		ifTrue: [
			self outdent.
			aTagWriter hasEmbeddedStringMarkup
				ifFalse: [self writeLineBreak]]! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/2/2014 00:36'!
formatBeforeEntityReference: anEntityReferenceWriter
	anEntityReferenceWriter isParameter
		ifTrue: [self writeLineBreak]! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:46'!
formatBeforeInternalSubset: aDoctypeDeclarationWriter
	self indent! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:41'!
formatBeforePI: aPiWriter
	aPiWriter isEmbeddedWithStringMarkup
		ifFalse: [self writeLineBreak]! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:50'!
formatBeforeStartTag: aTagWriter
	aTagWriter isEmbeddedWithStringMarkup
		ifFalse: [self writeLineBreak]! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 3/24/2015 19:34'!
formatBeforeXMLDeclaration: anXMLDeclarationWriter
	self writeLineBreak! !

!XMLWriterFormatter class methodsFor: 'instance creation' stamp: 'monty 3/24/2015 19:34'!
writer: aWriter
	^ self new setWriter: aWriter! !

!XMLWriterFormatter methodsFor: 'private' stamp: 'monty 11/30/2014 03:25'!
currentIndentLevel
	^ currentIndentLevel! !

!XMLWriterFormatter methodsFor: 'private' stamp: 'monty 11/30/2014 03:23'!
defaultIndentLevel
	^ 0! !

!XMLWriterFormatter methodsFor: 'private' stamp: 'monty 12/2/2014 04:24'!
defaultIndentString
	^ '    '! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:45'!
formatAfterDoctypeDeclaration: aDoctypeDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:42'!
formatAfterInternalSubset: aDoctypeDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:37'!
formatAfterStartTag: aTagWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:38'!
formatBeforeComment: aCommentWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:40'!
formatBeforeDeclaration: aDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:44'!
formatBeforeDoctypeDeclaration: aDoctypeDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:38'!
formatBeforeEndTag: aTagWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/2/2014 00:35'!
formatBeforeEntityReference: anEntityReferenceWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:46'!
formatBeforeInternalSubset: aDoctypeDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:41'!
formatBeforePI: aPiWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:38'!
formatBeforeStartTag: aTagWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 3/24/2015 19:34'!
formatBeforeXMLDeclaration: anXMLDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'private' stamp: 'monty 11/30/2014 03:25'!
indent
	currentIndentLevel := self currentIndentLevel + 1! !

!XMLWriterFormatter methodsFor: 'accessing' stamp: 'monty 11/30/2014 03:16'!
indentString
	^ indentString! !

!XMLWriterFormatter methodsFor: 'accessing' stamp: 'monty 11/30/2014 03:16'!
indentString: aString
	currentIndentLevel := 0.
	indentString := aString.! !

!XMLWriterFormatter methodsFor: 'private' stamp: 'monty 11/30/2014 03:25'!
outdent
	currentIndentLevel := self currentIndentLevel - 1! !

!XMLWriterFormatter methodsFor: 'initialization' stamp: 'monty 3/24/2015 19:34'!
setWriter: aWriter
	writer := aWriter.
	currentIndentLevel := self defaultIndentLevel.
	indentString := self defaultIndentString.! !

!XMLWriterFormatter methodsFor: 'private' stamp: 'monty 12/2/2014 00:50'!
writeIndent
	1 to: self currentIndentLevel do: [:i |
		self writer nextPutAll: self indentString]! !

!XMLWriterFormatter methodsFor: 'private' stamp: 'monty 12/13/2014 21:06'!
writeLineBreak
	self writer atBeginning
		ifFalse: [
			self writer nextPutAll: self writer lineBreak.
			(self currentIndentLevel > 0
				and: [self indentString notEmpty])
				ifTrue: [self writeIndent]]! !

!XMLWriterFormatter methodsFor: 'accessing' stamp: 'monty 11/30/2014 02:44'!
writer
	^ writer! !

!String methodsFor: '*XML-Writer-Core' stamp: 'monty 11/30/2014 06:06'!
embedInMarkupWriter: aMarkupWriter
	aMarkupWriter writer string: self! !

!Collection methodsFor: '*XML-Writer-Core' stamp: 'monty 11/30/2014 06:04'!
embedInMarkupWriter: aMarkupWriter
	self do: [:each | each embedInMarkupWriter: aMarkupWriter]! !
XMLWriter initialize!
